/*
 * The FML Forge Mod Loader suite.
 * Copyright (C) 2012 cpw
 *
 * This library is free software; you can redistribute it and/or modify it under the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
 * A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License along with this library; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */

package net.minecraftforge.fml.client;

import java.awt.Dimension;
import java.awt.image.BufferedImage;
import java.io.IOException;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.List;

import javax.imageio.ImageIO;

import net.minecraft.client.Minecraft;
import net.minecraft.client.gui.FontRenderer;
import net.minecraft.client.gui.GuiButton;
import net.minecraft.client.gui.GuiScreen;
import net.minecraft.client.gui.GuiTextField;
import net.minecraft.client.renderer.Tessellator;
import net.minecraft.client.renderer.WorldRenderer;
import net.minecraft.client.renderer.texture.DynamicTexture;
import net.minecraft.client.renderer.texture.TextureManager;
import net.minecraft.client.resources.I18n;
import net.minecraft.client.resources.IResourcePack;
import net.minecraft.util.ResourceLocation;
import net.minecraft.util.StringUtils;
import net.minecraftforge.fml.common.FMLLog;
import net.minecraftforge.fml.common.Loader;
import net.minecraftforge.fml.common.ModContainer;
import net.minecraftforge.fml.common.ModContainer.Disableable;

import org.apache.logging.log4j.Level;
import org.lwjgl.opengl.GL11;

import com.google.common.base.Strings;

/**
 * @author cpw
 *
 */
public class GuiModList extends GuiScreen
{
    private enum SortType
    {
        NORMAL(24), A_TO_Z(25), Z_TO_A(26);

        private int buttonID;

        private SortType(int buttonID)
        {
            this.buttonID = buttonID;
            this.comparator = new ModComparator(this);
        }
        
        private final ModComparator comparator;

        public static SortType getTypeForButton(GuiButton button)
        {
            for (SortType t : values()) {
                if (t.buttonID == button.field_146127_k) {
                    return t;
                }
            }
            return null;
        }
    }
    
    private static class ModComparator implements Comparator<ModContainer>
    {
        private SortType type;
        private ModComparator(SortType type)
        {
            this.type = type;
        }
        
        @Override
        public int compare(ModContainer o1, ModContainer o2)
        {
            String name1 = StringUtils.func_76338_a(o1.getName()).toLowerCase();
            String name2 = StringUtils.func_76338_a(o2.getName()).toLowerCase();
            switch(type)
            {
            case A_TO_Z:
                return name1.compareTo(name2);
            case Z_TO_A:
                return name2.compareTo(name1);
            default:
                return 0;
            }
        }
    }
    
    private GuiScreen mainMenu;
    private GuiSlotModList modList;
    private int selected = -1;
    private ModContainer selectedMod;
    private int listWidth;
    private ArrayList<ModContainer> mods;
    private GuiButton configModButton;
    private GuiButton disableModButton;
    private ResourceLocation cachedLogo;
    private Dimension cachedLogoDimensions;
    
    private int buttonMargin = 1;
    private int numButtons = SortType.values().length;

    private String lastFilterText = "";

    private GuiTextField search;
    private boolean sorted = false;
    private SortType sortType = SortType.NORMAL;

    /**
     * @param mainMenu
     */
    public GuiModList(GuiScreen mainMenu)
    {
        this.mainMenu=mainMenu;
        this.mods=new ArrayList<ModContainer>();
        FMLClientHandler.instance().addSpecialModEntries(mods);
        // Add child mods to their parent's list
        for (ModContainer mod : Loader.instance().getModList()) {
            if (mod.getMetadata()!=null && mod.getMetadata().parentMod==null && !Strings.isNullOrEmpty(mod.getMetadata().parent)) {
                String parentMod = mod.getMetadata().parent;
                ModContainer parentContainer = Loader.instance().getIndexedModList().get(parentMod);
                if (parentContainer != null)
                {
                    mod.getMetadata().parentMod = parentContainer;
                    parentContainer.getMetadata().childMods.add(mod);
                    continue;
                }
            }
            else if (mod.getMetadata()!=null && mod.getMetadata().parentMod!=null)
            {
                continue;
            }
            mods.add(mod);
        }
    }

    @SuppressWarnings("unchecked")
    @Override
    public void func_73866_w_()
    {
        for (ModContainer mod : mods) {
            listWidth=Math.max(listWidth,getFontRenderer().func_78256_a(mod.getName()) + 10);
            listWidth=Math.max(listWidth,getFontRenderer().func_78256_a(mod.getVersion()) + 10);
        }
        listWidth=Math.min(listWidth, 150);
        this.modList=new GuiSlotModList(this, mods, listWidth);
        this.modList.registerScrollButtons(this.field_146292_n, 7, 8);
        
        this.field_146292_n.add(new GuiButton(6, ((modList.right + this.field_146294_l) / 2) - 100, this.field_146295_m - 38, I18n.func_135052_a("gui.done")));
        configModButton = new GuiButton(20, 10, this.field_146295_m - 49, this.listWidth, 20, "Config");
        disableModButton = new GuiButton(21, 10, this.field_146295_m - 27, this.listWidth, 20, "Disable");
        this.field_146292_n.add(configModButton);
        this.field_146292_n.add(disableModButton);
        
        search = new GuiTextField(0, getFontRenderer(), 12, modList.bottom + 17, modList.listWidth - 4, 14);
        search.func_146195_b(true);
        search.func_146205_d(true);

        int width = (modList.listWidth / numButtons);
        int x = 10, y = 10;
        GuiButton normalSort = new GuiButton(SortType.NORMAL.buttonID, x, y, width - buttonMargin, 20, I18n.func_135052_a("fml.menu.mods.normal"));
        normalSort.field_146124_l = false;
        field_146292_n.add(normalSort);
        x += width + buttonMargin;
        field_146292_n.add(new GuiButton(SortType.A_TO_Z.buttonID, x, y, width - buttonMargin, 20, "A-Z"));
        x += width + buttonMargin;
        field_146292_n.add(new GuiButton(SortType.Z_TO_A.buttonID, x, y, width - buttonMargin, 20, "Z-A"));
    }
    
    @Override
    protected void func_73864_a(int x, int y, int button) throws IOException
    {
        super.func_73864_a(x, y, button);
        search.func_146192_a(x, y, button);
        if (button == 1 && x >= search.field_146209_f && x < search.field_146209_f + search.field_146218_h && y >= search.field_146210_g && y < search.field_146210_g + search.field_146219_i) {
            search.func_146180_a("");
        }
    }

    @Override
    protected void func_73869_a(char c, int keyCode) throws IOException
    {
        super.func_73869_a(c, keyCode);
        search.func_146201_a(c, keyCode);
    }
    
    @Override
    public void func_73876_c()
    {
        super.func_73876_c();
        search.func_146178_a();

        if (!search.func_146179_b().equals(lastFilterText))
        {
            reloadMods();
            sorted = false;
        }

        if (!sorted)
        {
            reloadMods();
            Collections.sort(mods, sortType.comparator);
            selected = modList.selectedIndex = mods.indexOf(selectedMod);
            sorted = true;
        }
    }

    private void reloadMods()
    {
        ArrayList<ModContainer> mods = modList.getMods();
        mods.clear();
        for (ModContainer m : Loader.instance().getActiveModList())
        {
            // If it passes the filter, and is not a child mod
            if (m.getName().toLowerCase().contains(search.func_146179_b().toLowerCase()) && m.getMetadata().parentMod == null)
            {
                mods.add(m);
            }
        }
        this.mods = mods;
        lastFilterText = search.func_146179_b();
    }

    @Override
    protected void func_146284_a(GuiButton button) throws IOException {
        if (button.field_146124_l)
        {
            SortType type = SortType.getTypeForButton(button);

            if (type != null)
            {
                for (GuiButton b : (List<GuiButton>) field_146292_n)
                {
                    if (SortType.getTypeForButton(b) != null)
                    {
                        b.field_146124_l = true;
                    }
                }
                button.field_146124_l = false;
                sorted = false;
                sortType = type;
                this.mods = modList.getMods();
            }
            else
            {
                switch (button.field_146127_k)
                {
                case 6:
                    this.field_146297_k.func_147108_a(this.mainMenu);
                    return;
                case 20:
                    try
                    {
                        IModGuiFactory guiFactory = FMLClientHandler.instance().getGuiFactoryFor(selectedMod);
                        GuiScreen newScreen = guiFactory.mainConfigGuiClass().getConstructor(GuiScreen.class).newInstance(this);
                        this.field_146297_k.func_147108_a(newScreen);
                    }
                    catch (Exception e)
                    {
                        FMLLog.log(Level.ERROR, e, "There was a critical issue trying to build the config GUI for %s", selectedMod.getModId());
                    }
                    return;
                }
            }
        }
        super.func_146284_a(button);
    }

    public int drawLine(String line, int offset, int shifty)
    {
        this.field_146289_q.func_78276_b(line, offset, shifty, 0xd7edea);
        return shifty + 10;
    }

    @Override
    public void func_73863_a(int p_571_1_, int p_571_2_, float p_571_3_)
    {
        this.modList.drawScreen(p_571_1_, p_571_2_, p_571_3_);
        this.func_73732_a(this.field_146289_q, "Mod List", this.field_146294_l / 2, 16, 0xFFFFFF);
        int offset = this.listWidth  + 20;
        if (selectedMod != null) {
            GL11.glEnable(GL11.GL_BLEND);
            if (!selectedMod.getMetadata().autogenerated) {
                configModButton.field_146125_m = true;
                disableModButton.field_146125_m = true;
                disableModButton.packedFGColour = 0xFF3377;
                configModButton.field_146124_l = false;
                int shifty = 35;
                String logoFile = selectedMod.getMetadata().logoFile;
                if (!logoFile.isEmpty())
                {
                    GL11.glColor4f(1.0F, 1.0F, 1.0F, 1.0F);
                    TextureManager tm = field_146297_k.func_110434_K();
                    IResourcePack pack = FMLClientHandler.instance().getResourcePackFor(selectedMod.getModId());
                    try
                    {
                        if (cachedLogo == null)
                        {
                            BufferedImage logo = null;
                            if (pack!=null)
                            {
                                logo = pack.func_110586_a();
                            }
                            else
                            {
                                InputStream logoResource = getClass().getResourceAsStream(logoFile);
                                if (logoResource != null)
                                {
                                    logo = ImageIO.read(logoResource);
                                }
                            }
                            if (logo != null)
                            {
                                cachedLogo = tm.func_110578_a("modlogo", new DynamicTexture(logo));
                                cachedLogoDimensions = new Dimension(logo.getWidth(), logo.getHeight());
                            }
                        }
                        if (cachedLogo != null)
                        {
                            this.field_146297_k.field_71446_o.func_110577_a(cachedLogo);
                            double scaleX = cachedLogoDimensions.width / 200.0;
                            double scaleY = cachedLogoDimensions.height / 65.0;
                            double scale = 1.0;
                            if (scaleX > 1 || scaleY > 1)
                            {
                                scale = 1.0 / Math.max(scaleX, scaleY);
                            }
                            cachedLogoDimensions.width *= scale;
                            cachedLogoDimensions.height *= scale;
                            int top = 32;
                            Tessellator tess = Tessellator.func_178181_a();
                            WorldRenderer world = tess.func_178180_c();
                            world.func_178970_b();
                            world.func_178985_a(offset,                               top + cachedLogoDimensions.height,  field_73735_i, 0, 1);
                            world.func_178985_a(offset + cachedLogoDimensions.width,  top + cachedLogoDimensions.height,  field_73735_i, 1, 1);
                            world.func_178985_a(offset + cachedLogoDimensions.width,  top,                                field_73735_i, 1, 0);
                            world.func_178985_a(offset,                               top,                                field_73735_i, 0, 0);
                            tess.func_78381_a();

                            shifty += 65;
                        }
                    }
                    catch (IOException e)
                    {
                        ;
                    }
                }
                this.field_146289_q.func_175063_a(selectedMod.getMetadata().name, offset, shifty, 0xFFFFFF);
                shifty += 12;

                shifty = drawLine(String.format("Version: %s (%s)", selectedMod.getDisplayVersion(), selectedMod.getVersion()), offset, shifty);
                shifty = drawLine(String.format("Mod ID: '%s' Mod State: %s", selectedMod.getModId(), Loader.instance().getModState(selectedMod)), offset, shifty);
                if (!selectedMod.getMetadata().credits.isEmpty()) {
                   shifty = drawLine(String.format("Credits: %s", selectedMod.getMetadata().credits), offset, shifty);
                }
                shifty = drawLine(String.format("Authors: %s", selectedMod.getMetadata().getAuthorList()), offset, shifty);
                shifty = drawLine(String.format("URL: %s", selectedMod.getMetadata().url), offset, shifty);
                shifty = drawLine(selectedMod.getMetadata().childMods.isEmpty() ? "No child mods for this mod" : String.format("Child mods: %s", selectedMod.getMetadata().getChildModList()), offset, shifty);
                int rightSide = this.field_146294_l - offset - 20;
                if (rightSide > 20)
                {
                    this.getFontRenderer().func_78279_b(selectedMod.getMetadata().description, offset, shifty + 10, rightSide, 0xDDDDDD);
                }
                Disableable disableable = selectedMod.canBeDisabled();
                if (disableable == Disableable.RESTART)
                {
                    disableModButton.field_146124_l = true;
                    disableModButton.field_146125_m = true;
                    disableModButton.packedFGColour = 0xFF3377;
                }
                else if (disableable == Disableable.YES)
                {
                    disableModButton.field_146124_l = true;
                    disableModButton.field_146125_m = true;
                    disableModButton.packedFGColour = 0;
                }
                else
                {
                    disableModButton.packedFGColour = 0;
                    disableModButton.field_146125_m = true;
                    disableModButton.field_146124_l = false;
                }
                IModGuiFactory guiFactory = FMLClientHandler.instance().getGuiFactoryFor(selectedMod);
                if (guiFactory == null || guiFactory.mainConfigGuiClass() == null)
                {
                    configModButton.field_146125_m = true;
                    configModButton.field_146124_l = false;
                }
                else
                {
                    configModButton.field_146125_m = true;
                    configModButton.field_146124_l = true;
                }
            } else {
                offset = ( this.listWidth + this.field_146294_l ) / 2;
                this.func_73732_a(this.field_146289_q, selectedMod.getName(), offset, 35, 0xFFFFFF);
                this.func_73732_a(this.field_146289_q, String.format("Version: %s",selectedMod.getVersion()), offset, 45, 0xFFFFFF);
                this.func_73732_a(this.field_146289_q, String.format("Mod State: %s",Loader.instance().getModState(selectedMod)), offset, 55, 0xFFFFFF);
                this.func_73732_a(this.field_146289_q, "No mod information found", offset, 65, 0xDDDDDD);
                this.func_73732_a(this.field_146289_q, "Ask your mod author to provide a mod mcmod.info file", offset, 75, 0xDDDDDD);
                configModButton.field_146125_m = false;
                disableModButton.field_146125_m = false;
            }
            GL11.glDisable(GL11.GL_BLEND);
        }
        else
        {
            configModButton.field_146125_m = false;
            disableModButton.field_146125_m = false;
        }
        super.func_73863_a(p_571_1_, p_571_2_, p_571_3_);

        String text = I18n.func_135052_a("fml.menu.mods.search");
        int x = ((10 + modList.right) / 2) - (getFontRenderer().func_78256_a(text) / 2);
        getFontRenderer().func_78276_b(text, x, modList.bottom + 5, 0xFFFFFF);
        search.func_146194_f();
    }

    Minecraft getMinecraftInstance() {
        return field_146297_k;
    }

    FontRenderer getFontRenderer() {
        return field_146289_q;
    }

    /**
     * @param var1
     */
    public void selectModIndex(int var1)
    {
        this.selected=var1;
        if (var1>=0 && var1<=mods.size()) {
            this.selectedMod=mods.get(selected);
        } else {
            this.selectedMod=null;
        }
        cachedLogo = null;
    }

    public boolean modIndexSelected(int var1)
    {
        return var1==selected;
    }
}
